@@ -1,4 +1,4 @@
-#!/usr/global/paper/bin/python
+#!/usr/bin/env python
 """
 A script for fitting parameters of a measurement equation given 
 starting parameters in a cal file and a list of sources.  The fitter used
@@ -34,12 +34,13 @@ o.add_option('--master', dest='master',
     help='Operate in master mode, employing daemon-mode servers to do the work and collecting the results.  Should be a comma delimited list of host:daemonid pairs to contact.  Daemon ID will be added to baseport to determine actual port used for TCP transactions.')
 o.add_option('--sim_autos', dest='sim_autos', action='store_true',
     help='Use auto-correlations in fitting.  Default is to use only cross-correlations.')
+o.add_option('--minuv',dest='minuv',default=0.,type='float',help='Minimum baseline length (in wavelengths at 150 MHz) to consider.')
 
 opts, args = o.parse_args(sys.argv[1:])
 
 # Parse command-line options
 uv = a.miriad.UV(args[0])
-opts.ant += ',cross'
+#opts.ant += ',cross'
 a.scripting.uv_selector(uv, opts.ant, opts.pol)
 aa = a.cal.get_aa(opts.cal, uv['sdf'], uv['sfreq'], uv['nchan'])
 chans = a.scripting.parse_chans(opts.chan, uv['nchan'])
@@ -85,6 +86,8 @@ first_fit = None    # Used to normalize fit values to the starting fit
 mfq = cat.get('mfreq')
 dbuf = None
 
+def uvlen(A): return n.sqrt(n.dot(A,A))
+
 # The function to be optimized
 def fit_func(prms, filelist, decimate, decphs):
     global first_fit, dbuf
@@ -97,8 +100,9 @@ def fit_func(prms, filelist, decimate, decphs):
             if not prms.has_key(k2): prms[k2] = {}
             for sp in sprm:
                 prms[k2][sp] = prms[k][sp]
-    if not opts.quiet: a.fit.print_params(prms)
-    print prms
+    if not opts.quiet:
+        a.fit.print_params(prms)
+        print prms
     aa.set_params(prms)
     cat.set_params(prms)
     a1,a2,th = cat.get('srcshape')
@@ -113,21 +117,23 @@ def fit_func(prms, filelist, decimate, decphs):
             a.scripting.uv_selector(uv, opts.ant, opts.pol)
             uv.select('decimate', decimate, decphs)
             for (uvw,t,(i,j)),d,f in uv.all(raw=True):
-                if not dbuf.has_key(t): dbuf[t] = {}
-                if not opts.sim_autos and i == j: continue
                 bl = a.miriad.ij2bl(i,j)
+                if not dbuf.has_key(t): dbuf[t] = {}
+                if not dbuf[t].has_key(bl): dbuf[t][bl] = {}
+                if not opts.sim_autos and i == j: continue
+                if uvlen(aa.get_baseline(i,j))*0.15 < opts.minuv: continue
                 d = d.take(chans)
                 f = f.take(chans)
-                dbuf[t][bl] = (d, f, 
-                        n.where(f, 0, n.abs(d)**2).sum(),
-                        a.miriad.pol2str[uv['pol']])
+                pol = a.miriad.pol2str[uv['pol']]
+                dbuf[t][bl][pol] = (d, f, n.where(f, 0, n.abs(d)**2).sum())
         if not opts.quiet:
             samp, vsamp, wgts = 0, 0, 0.
             for t in dbuf:
               for bl in dbuf[t]:
-                samp += len(dbuf[t][bl][1])
-                vsamp += n.logical_not(dbuf[t][bl][1]).astype(n.int).sum()
-                wgts += dbuf[t][bl][2]
+                for pol in dbuf[t][bl]:
+                    samp += len(dbuf[t][bl][pol][1])
+                    vsamp += n.logical_not(dbuf[t][bl][pol][1]).astype(n.int).sum()
+                    wgts += dbuf[t][bl][pol][2]
             print 'Cache summary:'
             print '   %d samples' % samp
             print '   %d valid' % vsamp
@@ -144,12 +150,14 @@ def fit_func(prms, filelist, decimate, decphs):
             ionrefs=(dra,ddec), srcshapes=(a1,a2,th))
         for bl in dbuf[t]:
             i,j = a.miriad.bl2ij(bl)
-            d,f,nsamp,pol = dbuf[t][bl]
-            sim_d = aa.sim(i, j, pol=pol)
-            difsq = n.abs(d - sim_d)**2
-            difsq = n.where(f, 0, difsq)
-            score += difsq.sum()
-            nsamples += nsamp
+            for pol in dbuf[t][bl]:
+                d,f,nsamp = dbuf[t][bl][pol]
+                aa.set_active_pol(pol)
+                sim_d = aa.sim(i, j)
+                difsq = n.abs(d - sim_d)**2
+                difsq = n.where(f, 0, difsq)
+                score += difsq.sum()
+                nsamples += nsamp
     if opts.daemon: return score, nsamples
     if nsamples == 0:
         first_fit = 0.
@@ -278,6 +286,7 @@ else:
             prms,score = rv[:2]
             prms = a.fit.reconstruct_prms(prms, key_list)
             print
+            print prms
             a.fit.print_params(prms)
             print 'Score:', score * first_fit, 
             print '(%2.2f%% of %f)' % (100 * score, first_fit)
