@@ -43,6 +43,7 @@ def mfunc(uv, p, d, f):
     global curtime, src_spec
     uvw,t,(i,j) = p
     pol = a.miriad.pol2str[uv['pol']]
+    aa.set_active_pol(pol)
     if t != curtime:
         curtime = t
         aa.set_jultime(t)
@@ -51,9 +52,10 @@ def mfunc(uv, p, d, f):
             src_spec = s.get_jys()
             s_eq = cat.get_crds('eq', ncrd=3)
             aa.sim_cache(s_eq)
-    if opts.passband: passband = aa.passband(i,j)
+    if opts.passband:
+        passband = aa.passband(i,j)
     else: passband = 1
-    if opts.beam: bm_resp = aa.bm_response(i,j,pol=pol).squeeze()
+    if opts.beam: bm_resp = aa.bm_response(i,j).squeeze()
     else: bm_resp = 1
     gain = passband * bm_resp * src_spec
     d /= n.where(gain == 0, 1, gain)
