@@ -101,6 +101,7 @@ curtime = None
 def mdl(uv, p, d, f):
     global curtime, eqs
     uvw, t, (i,j) = p
+    pol = a.miriad.pol2str[uv['pol']]
     if i == j: return p, d, f
     if curtime != t:
         curtime = t
@@ -120,7 +121,8 @@ def mdl(uv, p, d, f):
         #ind = n.concatenate(ind)
         aa.sim_cache(eqs, flx, mfreqs=mfq, 
             ionrefs=(dras,ddecs), srcshapes=(a1s,a2s,ths))
-    sd = aa.sim(i, j, pol=a.miriad.pol2str[uv['pol']])
+    aa.set_active_pol(pol)
+    sd = aa.sim(i, j)
     if opts.mode.startswith('sim'):
         d = sd
         if not opts.flag: f = no_flags
@@ -135,7 +137,7 @@ def mdl(uv, p, d, f):
         noise_amp = n.random.random(d.shape) * opts.noiselev
         noise_phs = n.random.random(d.shape) * 2*n.pi * 1j
         noise = noise_amp * n.exp(noise_phs)
-        d += noise * aa.passband(i, j)
+        d += noise * aa.passband(i,j)
     return p, n.where(f, 0, d), f
 
 if len(args) > 0:
@@ -150,15 +152,13 @@ if len(args) > 0:
         a.scripting.uv_selector(uvi, opts.ant)
         uvo = a.miriad.UV(uvofile, status='new')
         uvo.init_from_uv(uvi)
-        uvo.pipe(uvi, mfunc=mdl, raw=True,
-            append2hist="MDLVIS: srcs=%s mode=%s flag=%s noise=%f\n" % \
-                (opts.src, opts.mode, opts.flag, opts.noiselev))
+        uvo.pipe(uvi, mfunc=mdl, raw=True, append2hist="MDLVIS: " + ' '.join(sys.argv) + '\n')
 else:
     # Initialize a new UV file
     pols = opts.pol.split(',')
     uv = a.miriad.UV('new.uv', status='new')
     uv._wrhd('obstype','mixed-auto-cross')
-    uv._wrhd('history','MDLVIS: created file.\nMDLVIS: srcs=%s mode=%s flag=%s noise=%f\n' % (opts.src, opts.mode, opts.flag, opts.noiselev))
+    uv._wrhd('history','MDLVIS: created file.\nMDLVIS: ' + ' '.join(sys.argv) + '\n')
     uv.add_var('telescop','a'); uv['telescop'] = 'AIPY'
     uv.add_var('operator','a'); uv['operator'] = 'AIPY'
     uv.add_var('version' ,'a'); uv['version'] = '0.0.1'
@@ -203,10 +203,15 @@ else:
         uv['lst'] = aa.sidereal_time()
         uv['ra'] = aa.sidereal_time()
         uv['obsra'] = aa.sidereal_time()
+        # XXX should really honor opts.ants for which antennas to simulate here.
         for i,ai in enumerate(aa):
             for j,aj in enumerate(aa):
+                try:
+                    if ai.pol+aj.pol not in pols: continue
+                    i,j = ai.num,aj.num
+                except(AttributeError): pass
                 if j < i: continue
-                crd = ai.pos - aj.pos
+                crd = aa.get_baseline(i,j)
                 preamble = (crd, t, (i,j))
                 for pol in pols:
                     uv['pol'] = a.miriad.str2pol[pol]
