@@ -27,6 +27,8 @@ o.add_option('--fmt', dest='fmt', default='im%04d',
     help='A format string for counting successive images written to files.  Default is im%04d (i.e. im0001).')
 o.add_option('--skip_phs', dest='skip_phs', action='store_true',
     help='Do not phase visibilities before gridding.')
+o.add_option('--zen_phased', dest='zen_phased', action='store_true',
+    help='If data have already have already been phased to zenith, set this flag.')
 o.add_option('--skip_amp', dest='skip_amp', action='store_true',
     help='Do not use amplitude information to normalize visibilities.')
 o.add_option('--skip_bm', dest='skip_bm', action='store_true',
@@ -56,6 +58,7 @@ chans = a.scripting.parse_chans(opts.chan, uv['nchan'])
 a.scripting.uv_selector(uv, opts.ant, opts.pol)
 aa = a.cal.get_aa(opts.cal, uv['sdf'], uv['sfreq'], uv['nchan'])
 aa.select_chans(chans)
+aa.set_active_pol(opts.pol)
 afreqs = aa[0].beam.afreqs
 cfreq = n.average(afreqs)
 aa.set_jultime(t)
@@ -171,6 +174,7 @@ for srccnt, s in enumerate(cat.values()):
       uv.select('decimate', opts.decimate, opts.decphs)
       # Read all data from each file
       for (crd,t,(i,j)),d,f in uv.all(raw=True):
+          pol = a.miriad.pol2str[uv['pol']]
           history = uv['history']
           history = history +  sys.argv[0].split('/')[-1].strip()+' ' + ' '.join(sys.argv[1:])
           if curtime != t:
@@ -204,15 +208,18 @@ for srccnt, s in enumerate(cat.values()):
               s_eq = src.get_crds('eq', ncrd=3)
               aa.sim_cache(s_eq)
           if s.alt < opts.altmin * a.img.deg2rad: continue
+          aa.set_active_pol(pol)
           d,f = d.take(chans), f.take(chans)
           if not opts.skip_amp: d /= aa.passband(i,j)
           # Throws PointingError if not up:
-          if not opts.skip_phs: d = aa.phs2src(d, s, i, j)
+          if not opts.skip_phs:
+              if opts.zen_phased: d = aa.unphs2src(d, 'z', i, j)
+              d = aa.phs2src(d, s, i, j)
           u,v,w = aa.gen_uvw(i,j,src=s)
           longenough = n.where(n.sqrt(u**2+v**2) < opts.minuv, 0, 1).squeeze()
           if not opts.skip_bm:
               # Calculate beam strength for weighting purposes
-              wgt = aa.bm_response(i,j,pol=opts.pol).squeeze()
+              wgt = aa.bm_response(i,j).squeeze()
               # Optimal SNR: down-weight beam-attenuated data 
               # by another factor of the beam response.
               d *= wgt; wgt *= wgt
