@@ -6,12 +6,11 @@ Healpix FITS format) from individual "flat" maps stored in FITS files.
 Author: Aaron Parsons
 """
 
-import sys, numpy as n, os, aipy as a, optparse
+import sys, numpy as n, os, aipy as a, optparse, ephem
 
 o = optparse.OptionParser()
 o.set_usage('mk_map.py [options] *.fits')
 o.set_description(__doc__)
-a.scripting.add_standard_options(o, pol=True, chan=True)
 o.add_option('-i', '--interpolate', dest='interpolate', action='store_true',
     help='Use sub-pixel interpolation when gridding data to healpix map.')
 o.add_option('-m', '--map', dest='map',
@@ -31,12 +30,22 @@ prev_dra = None
 for i, filename in enumerate(args):
     img, kwds = a.img.from_fits(filename)
     img = img.squeeze()
-    s = a.ant.RadioFixedBody(kwds['ra']*a.img.deg2rad,
-                             kwds['dec']*a.img.deg2rad)
+    # Read ra/dec of image center, which are stored in J2000
+    assert(kwds['epoch'] == 2000)
+    s = ephem.Equatorial(kwds['ra']*a.img.deg2rad, kwds['dec']*a.img.deg2rad, 
+        epoch=ephem.J2000)
+    # To precess the entire image to J2000, we actually need to precess the
+    # coords of the center back to the epoch of the observation (obs_date),
+    # get the pixel coordinates in that epoch, and then precess the coords of
+    # each pixel.  This is because extrapolating pixel coords from the J2000 
+    # center assumes the J2000 ra/dec axes, which may be tilted relative to
+    # the ra/dec axes of the epoch of the image.
+    s = ephem.Equatorial(s, epoch=kwds['obs_date'])
+    ra, dec = s.get()
     print '-----------------------------------------------------------'
     print 'Reading file %s (%d / %d)' % (filename, i + 1, len(args))
     print kwds
-    print 'Pointing (ra, dec):', s._ra, s._dec
+    print 'Pointing (ra, dec):', ra, dec
     print 'Image Power:', n.abs(img).sum()
     if prev_dra != kwds['d_ra']:
         prev_dra = kwds['d_ra']
@@ -49,10 +58,14 @@ for i, filename in enumerate(args):
         map_wgts.shape = (map_wgts.size,)
         valid = n.logical_not(map_wgts.mask)
         map_wgts = map_wgts.compress(valid)
-    # Get coordinates of image pixels in original (J2000) epoch
-    ex,ey,ez = im.get_eq(s._ra, s._dec, center=(DIM/2,DIM/2))
+    # Get coordinates of image pixels in the epoch of the observation
+    ex,ey,ez = im.get_eq(ra, dec, center=(DIM/2,DIM/2))
     ex = ex.compress(valid); ey = ey.compress(valid); ez = ez.compress(valid)
     img = img.flatten(); img = img.compress(valid)
+    # Precess the pixel coordinates to the (J2000) epoch of the map
+    m = a.coord.convert_m('eq','eq', 
+        iepoch=kwds['obs_date'], oepoch=ephem.J2000)
+    ex,ey,ez = n.dot(m, n.array([ex,ey,ez])) 
     # Put the data into the skymap
     skymap.add((ex,ey,ez), map_wgts, img)
 skymap.to_fits(opts.map, clobber=True)
