@@ -12,6 +12,8 @@ o.set_description(__doc__)
 a.scripting.add_standard_options(o, cal=True, src=True)
 o.add_option('--setphs', dest='setphs', action='store_true',
     help='Instead of rotating phase, assign a phase corresponding to the specified source.')
+o.add_option('--rot_uvw',action='store_true',
+    help='Rotate the uvw coordinates to the source. Useful for exporting beyond AIPY')
 opts,args = o.parse_args(sys.argv[1:])
 
 # Parse command-line options
@@ -30,6 +32,8 @@ curtime = None
 def phs(uv, p, d, f):
     global curtime
     uvw, t, (i,j) = p
+    pol = a.miriad.pol2str[uv['pol']]
+    aa.set_active_pol(pol)
     if curtime != t:
         curtime = t
         aa.set_jultime(t)
@@ -39,6 +43,9 @@ def phs(uv, p, d, f):
         if opts.setphs: d = aa.unphs2src(n.abs(d), src, i, j)
         elif src is None: d *= n.exp(-1j*n.pi*aa.get_phs_offset(i,j))
         else: d = aa.phs2src(d, src, i, j)
+        if opts.rot_uvw: 
+            uvw = aa.gen_uvw(i,j,src=src)
+            p = (uvw,t,(i,j))
     except(a.phs.PointingError): d *= 0
     return p, d, f
 
