@@ -15,3 +15,7 @@ import ephem, fit, healpix, img
 import interp, cal, map, miriad
 import optimize, rfi, amp, scripting, src, _src, utils
 import dsp
+import pol, twodgauss #added by dfm
+from __gitlog__ import __gitlog__
+from __version__ import __version__
+from __branch__ import __branch__
