@@ -283,9 +283,16 @@ class UV(_miriad.UV):
 
 def bl2ij(bl):
     bl = int(bl)
-    return (bl>>8)-1, (bl&255) - 1
+    if (bl > 65536):
+        bl -= 65536
+        mant = 2048
+    else:
+        mant = 256
+#AAR    return (bl>>8)-1, (bl&255) - 1
+    return bl/mant - 1, bl%mant -1
 
 def ij2bl(i, j):
     if i > j: i,j = j,i
-    return (i+1)<<8 | (j+1)
+    if j + 1 < 256: return 256*(i+1) + (j+1)
+    else: return 2048*(i+1) + (j+1) + 65536
 
