@@ -3,6 +3,7 @@ Module for representing antenna array geometry and for generating
 phasing information.
 """
 import ephem, math, numpy as n, coord, const, _cephes
+from miriad import ij2bl, bl2ij
 
 class PointingError(Exception):
     """An error to throw if a source is below the horizon."""
@@ -38,6 +39,8 @@ class RadioBody:
         self.mfreq = mfreq
         self.ionref = list(ionref)
         self.srcshape = list(srcshape)
+    def __str__(self):
+        return "%s"% (self.src_name,)
     def compute(self, observer):
         """Update coordinates relative to the provided observer.  Must be
         called at each time step before accessing information."""
@@ -72,6 +75,9 @@ class RadioFixedBody(ephem.FixedBody, RadioBody):
         ephem.FixedBody.__init__(self)
         self._ra, self._dec = ra, dec
         self._epoch = epoch
+    def __str__(self):
+        if self._dec<0: return RadioBody.__str__(self) + '\t' + str(self._ra) +'\t'+ str(self._dec)
+        else: return RadioBody.__str__(self) + '\t' + str(self._ra) +'\t'+'+' + str(self._dec)
     def compute(self, observer):
         ephem.FixedBody.compute(self, observer)
         RadioBody.compute(self, observer)
@@ -264,12 +270,13 @@ class AntennaArray(ArrayLocation):
     def ij2bl(self, i, j):
         """Convert baseline i,j (0 indexed) to Miriad's (i+1) << 8 | (j+1) 
         indexing scheme."""
-        return (int(i)+1) << 8 | (int(j)+1)
+        return ij2bl(i,j)
+    
     def bl2ij(self, bl):
         """Convert Miriad's (i+1) << 8 | (j+1) baseline indexing scheme to 
         i,j (0 indexed)"""
-        bl = int(bl)
-        return ((bl >> 8) & 255) - 1, (bl & 255) - 1
+        return bl2ij(bl)
+
     def bl_indices(self, auto=True, cross=True):
         """Return bl indices for baselines in the array."""
         if auto:
